/********************************* (C) РОТЕК ***********************************
 * @module  common_config
 * @file    common_config.h
 * @version 1.0.0
 * @date    XX.XX.XXXX
 * $brief   Общий конфиг для всей периферии. Настройка состава проекта.
 *******************************************************************************
 * @history     Version  Author         Comment
 * XX.XX.XXXX   1.0.0    Telenkov D.A.  First release.
 *******************************************************************************
 */

/* Define to prevent recursive  ----------------------------------------------*/
#ifndef COMMON_CONFIG_H
#define COMMON_CONFIG_H


/**
  * @brief  Маркер ревизии платы для валидации *.bin файла
  */
#define HW_REV                          "BT-6702"

#define HW_REV_LEN                      16

#define HW_REV_OFFSET                   0x1F0

/**
  * @brief  Версия прошивки
  */
#define VERSION                         "1.0"
   
/**
  * @brief  Адрес сектора настроек
  */ 	
#define SETTINGS_SECTOR         0x08004000

/**
  * @brief  Адрес сектора настроек
  */ 	
#define CRC_ADDRESS             0x08005F9C

/**
  * @brief  Адрес сектора системных настроек
  */
#define SYS_SECTOR         		0x08008000

/**
  * @brief  Адрес CRC системных настроек
  */
#define SYS_CRC_ADDRESS         0x0800BF9C

/**
  * @brief  Слово для контроля целостности структуры настроек
  */ 	
#define SETTINGS_CONTROL_WORD   123456789

/**
  * @brief  Число попыток загрузки основной прошивки
  */
#define BOOT_TRY   6

/**
  * @brief  Таймаут возврата в основное ПО, сек.
  */
#define UPDATE_TIMEOUT  10*60

#define USER_FLASH_FIRST_PAGE_ADDRESS 0x08020000 /* Only as example see comment */
#define USER_FLASH_LAST_PAGE_ADDRESS  0x08060000
#define USER_FLASH_END_ADDRESS        0x0807FFFF
#define USER_FLASH_CRC_ADDRESS        0x0807FFFC

#define IAP_FLASH_FIRST_PAGE_ADDRESS  0x08000000 /* Only as example see comment */
#define IAP_FLASH_CRC_ADDRESS         0x0801FFFC
	
	
/**
  * @brief  Размер очереди для хранения трапов
  */
#define SNMP_TRAP_QUEUE_SIZE   100

/**
  * @brief  Отладочный порт USART и консоль
  */
#define USART_DEBUG_ENABLE

#define SYSTEMTICK_PERIOD_MS  1

/**
  * @brief  RTC clock
  */
#define RTC_ENABLE

    
/**
  * @brief  LED индикация
  */
#define LED_ENABLE

/**
  * @brief  Кнопки
  */
#define BUTTON_ENABLE

/**
  * @brief  Ethernet сеть
  */
#define NET_ENABLE

/**
  * @brief  Веб сервер
  */
#define WEB_SERVER_ENABLE

/**
  * @brief  SNMP протокол
  */
#if defined HARDWARE_BT6702
#define SNMP_ENABLE
#endif

/**
  * @brief  UPS
  */
#if defined HARDWARE_BT6702
#define UPS_ENABLE
#endif
    
/**
  * @brief  Мониторинг FreeRtos
  */
//#define DEBUG_FREERTOS
    


#endif /* #ifndef COMMON_CONFIG_H */

/********************************* (C) РОТЕК **********************************/
